@@ -23,7 +23,7 @@ def get_data_path():
     # Check for data next to the package.
     try:
         root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-        return os.path.join(root,'doc/_examples')
+        return os.path.join(root,'doc/examples')
     except:
         raise RuntimeError("Could not find sample data")
 
